#ifndef BILLBOARD_H_ZERO
#define BILLBOARD_H_ZERO
/*
   Billboard.h

   Written by Zero (xmaste00).

   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.
*/
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoTextureCoordinate2.h>
#include <Inventor/nodes/SoTransparencyType.h>

#include "Main.h"

class SgoManager;

class Billboard{
private:
	SoSeparator * Manager;
	SoSeparator * Root;
	SoRotation	* Rotation;
	SoTranslation * Translation;

	SoCoordinate3 * Coordinate;
	SoIndexedTriangleStripSet * TriangleStripSet;
	SoTexture2 * Texture;
	SoTextureCoordinate2 * TextureCoordinate;
	
	SoPerspectiveCamera * Camera;

	float Dimension;//rozmer billboardu
	float FPS;		//rychlost animace
	double lastTime;
	int TextureDimension;	//rozmer textury pro animaci 1=bez animace cislo 2^x
	int Frame;  // frame animace	od 1 do TD*TD
	bool Used;	// activni/neaktivni (viditelny/neviditelny)

	void updateDimension(float dimension);
	void updateTextureCoordinate(int frame);

public:
	Billboard();
	Billboard(float dimension, char * file);
	
	SoSeparator * getSceneGraph() const { return Root; }
	SoSeparator * getManager() const { return Manager; }
	void setManager(SoSeparator * manager) { Manager=manager; }
	
	void setOrientation(SbVec3f axis, float rad) { this->Rotation->rotation.setValue(axis,rad); }
	void setPos(SbVec3f pos) { this->Translation->translation.setValue(pos); }
	void setDimension(float d) { this->Dimension=d/2.0f; this->updateDimension(d/2.0f); }

	void setTexture(char * file, int d) { this->Texture->filename.setValue(file); this->TextureDimension=d;}
	void setTexture(char * file) { this->Texture->filename.setValue(file); }
	void setTextureDimension(int d) { this->TextureDimension=d; }
	void setTextureFPS(float fps) { this->FPS=fps; }

	bool isFree() { return !this->Used; }
	bool isUsed() { return this->Used; }

	void setCamera(SoPerspectiveCamera * camera) { this->Camera=camera; }
	void unrefCamera() { this->Camera=NULL; }

	void useBillboard();
	void useBillboard(SbVec3f pos);

	void timeStep(double time, double dt);
};

#endif /* BILLBOARD_H_ZERO */