#ifndef SG_OBJECT_H
#define SG_OBJECT_H
/*

   SgObject.h - base class for scene objects

   Written by Petr Mastera (xmaste00).


   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.

*/

#include "Main.h"

class SoNode;
class SoSeparator;
class SoMatrixTransform;
class SgoManager;
class SbVec3f;

class SgObject {
public:
  enum MODEL_SET { RENDER = 0x01, COLLISION = 0x02, ALL = 0x03 };
  enum TYPE_OBJECT { SUN = 0x01, PLANET = 0x02, OTHER = 0x03, CAMERA = 0x04 };

protected:
  // OpenInventor Variables
  SgoManager * Manager;
  SoSeparator * Root;
  SoMatrixTransform * MatrixTransform;
  
  // Model Variables
  SoNode * Model;
  SoNode * CollisionModel;
  SbVec3f Scale;				// for MatrixTanfosrm
  float CollisionRadius;
  float ActionRadius;

  // Move
  int durFB;
  int durLR;
  int durUD;
  int durX;
  int durY;
  int durZ;
  
  // Physical Variables
  float Mass;
  float CoefficientOfRestitution;
  SbVec3f Position;				// for MatrixTanfosrm
  SbVec3f OldPosition;
  SbVec3f CentrOfMass;
  SbVec3f Velocity;
  SbVec3f AngularVelocity;
  SbVec3f AngularMomentum;
  SbVec3f Force;
  //SbVec3f Torque;
  SbRotation Orientation;		// for MatrixTanfosrm
  SbRotation OldOrientation;
  SbRotation ScaleOrientation;	// for MatrixTanfosrm
  SbMatrix InertiaTensor;

  //identification
  TYPE_OBJECT Type;
  int ID;
  
  //Other Variables
  bool Select;

  //Metods
  void createHiddenScene();   // Creates internal scene and initializes root, matrixTransform, model and collisionModel members.
  void releaseHiddenScene();  // Releases internal scene.

  void updateMatrixTransform();

public:
  SgObject();
  SgObject(const char *model, const char *collisionModel = NULL,
           const SbVec3f &pos = SbVec3f(0.f,0.f,0.f), const SbVec3f &dir = SbVec3f(0.f,0.f,1.f),
           float radius = 0.f, float weight = 1.f);
  virtual ~SgObject();
  
  SgoManager * getManager() const { return Manager; }
  void setManager(SgoManager * manager) { Manager=manager; }
  SoSeparator * getSceneGraph() const { return Root; }
  SoMatrixTransform * getMatrixTransform() { return MatrixTransform; }
  const SbMatrix & getMatrix() const { return MatrixTransform->matrix.getValue(); }
  void setMatrix(const SbMatrix & m) { MatrixTransform->matrix=m; }
  SoNode * getModel() const { return Model; }
  SoNode * getCollisionModel() const { return CollisionModel; }
  void setModel(SoNode * model, MODEL_SET which = ALL);
  SbBool setModel(const char * filename, MODEL_SET which = ALL);
  void setScale(SbVec3f scale) { Scale=scale; }
  SbVec3f getScale() const { return Scale; }
  void setCollisionRadius(float radius) { CollisionRadius=radius; }
  float getCollisionRadius() const { return CollisionRadius; }
  void setActionRadius(float radius) { ActionRadius=radius; }
  float getActionRadius() const { return ActionRadius; }
  void setMass(float mass) { Mass=mass; }
  float getMass() const { return Mass; }
  void setCenterOfMass(SbVec3f com) { this->CentrOfMass=com; }
  SbVec3f getCenterOfMass() const { return this->CentrOfMass; }
  void setCoefficientOfRestitution(float e) { CoefficientOfRestitution=e; }
  float getCoefficientOfRestitution() const { return CoefficientOfRestitution; }
  void setType(TYPE_OBJECT t) { this->Type=t; }
  TYPE_OBJECT getType() { return this->Type; }
  void setID(int id) { this->ID=id; }
  int getID() { return this->ID; }

  void setForce(SbVec3f f) { this->Force=f; }
  SbVec3f getForce() { return this->Force; }


  //move
  void incDurFB() { durFB++; }
  void incDurLR() { durLR++; }
  void incDurUD() { durUD++; }
  void incDurX() { durX++; }
  void incDurY() { durY++; }
  void incDurZ() { durZ++; }
  void decDurFB() { durFB--; }
  void decDurLR() { durLR--; }
  void decDurUD() { durUD--; }
  void decDurX() { durX--; }
  void decDurY() { durY--; }
  void decDurZ() { durZ--; }
  bool zeroDurFB() { if (durFB) return false; else return true; }
  bool zeroDurLR() { if (durLR) return false; else return true; }
  bool zeroDurUD() { if (durUD) return false; else return true; }
  bool zeroDurX() { if (durX) return false; else return true; }
  bool zeroDurY() { if (durY) return false; else return true; }
  bool zeroDurZ() { if (durZ) return false; else return true; }

  //position
  SbVec3f getDirVel() const;
  SbVec3f getDirVec() const;
  SbVec3f getUpVec() const;
  SbVec3f getLeftVec() const;
  SbVec3f& getPos() { return this->Position; }
  void setPos(const SbVec3f &p) { this->Position=p; }
  void setPos(const float x, const float y, const float z) { this->Position=SbVec3f(x,y,z); }
  void updPos(const SbVec3f &deltaP) { this->Position+=deltaP; }
  void updPos(const float x, const float y, const float z) { this->Position+=SbVec3f(x,y,z); }
  SbVec3f& getOldPos() { return this->OldPosition; }
  void setOldPos() { this->Position=this->OldPosition; }
  //velocity
  SbVec3f& getVel() { return this->Velocity; }
  void setVel(const SbVec3f &v) { this->Velocity=v; }
  void setVel(const float x, const float y, const float z) { this->Velocity=SbVec3f(x,y,z); }
  SbVec3f& getAngVel() { return this->AngularVelocity; }
  void setAngVel(const SbVec3f &av) { this->AngularVelocity=av; }
  void setAngVel(const float x, const float y, const float z) { this->AngularVelocity=SbVec3f(x,y,z); }
  SbVec3f& getAngMom() { return this->AngularMomentum; }
  void setAngMom(const SbVec3f &am) { this->AngularMomentum=am; }
  void setAngMom(const float x, const float y, const float z) { this->AngularMomentum=SbVec3f(x,y,z); }
  //rotation
  SbRotation getOrientation() { return this->Orientation; }
  void setOrientation(const SbRotation &r) { this->Orientation=r; }
  void setOrientation(const SbVec3f &axis, const float radians) { this->Orientation=SbRotation(axis,radians); }
  void updOrientation(const SbVec3f &axis, const float radians) { this->Orientation*=SbRotation(axis,radians); }
  void RotLR(const float radians) { this->Orientation*=SbRotation(this->getUpVec(),radians); }	//LeftRight //otaceni vlevo vpravo
  void RotUD(const float radians) { this->Orientation*=SbRotation(this->getLeftVec(),radians); }//UpDown	//otaceni nahoru dolu
  void RotAR(const float radians) { this->Orientation*=SbRotation(this->getDirVec(),radians); }	//all round	//otaceni kolem dokola v ose pohybu
  void TorqueLR(const float torque) { this->AngularMomentum+=this->getLeftVec().cross(this->getDirVec()*torque); }	//Left/Rignht//otaceni vlevo vpravo
  void TorqueUD(const float torque) { this->AngularMomentum+=this->getDirVec().cross(this->getUpVec()*torque); }	//Up/Down	//otaceni nahoru dolu
  void TorqueAR(const float torque) { this->AngularMomentum+=this->getLeftVec().cross(this->getUpVec()*torque); }	//all round	//otaceni kolem dokola v ose pohybu
  SbRotation& getOldOrientation() { return this->OldOrientation; }
  void setOldOrientation() { this->Orientation=this->OldOrientation; }

  //InertiaTensor
  SbMatrix getInertiaTensor() { return this->InertiaTensor; }
  void setInertiaTensor(const SbMatrix &m) { this->Orientation=m; }

  //speed,direction
  const SbVec3f& getSpeed() const  { return this->Velocity; }
  void setSpeed(const SbVec3f &speed) { this->Velocity=speed; }
  void setSpeed(const float x, const float y, const float z) { this->Velocity=SbVec3f(x,y,z); }

  virtual void timeStep(double time, double dt);
};

#endif /* SG_OBJECT_H */
