/*

   SgoManager.cpp - scene objects manager class

   Written by Petr Mastera (xmaste00) and PCJohn (peciva).


   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.

*/
#include "SgoManager.h"


SgoManager::SgoManager()
{
  sgoRoot = new SoSeparator;
  sgoRoot->ref();

  this->CollidingSgos.Collision=false;
  this->epsilon=0.0000001f;
	
  collisionRoot = new SoGroup;
  collisionRoot->ref();
  collisionRoot->addChild(new SoSeparator);
  collisionRoot->addChild(new SoSeparator);

  ida = new SoIntersectionDetectionAction;
  ida->setManipsEnabled(FALSE);
  ida->setDraggersEnabled(FALSE);
  ida->setIntersectionDetectionEpsilon(0.0f);
  //ida->setFilterCallback(filterCB, NULL);
  ida->addIntersectionCallback(intersectionCb, this);
}
SgoManager::~SgoManager()
{
  sgoRoot->unref();
  collisionRoot->unref();
}
void SgoManager::appendSgo(SgObject *sgo)
{
  if (sgo->getManager()) {
    if (sgo->getManager() == this)  return;
    else  removeSgo(sgo);
  }
  sgoList.append(sgo);
  sgoRoot->addChild(sgo->getSceneGraph());
  sgo->setManager(this);
}
void SgoManager::removeSgo(SgObject *sgo)
{
  int i = sgoList.find(sgo);
  assert(i != -1 && "Object not present.");
  removeSgo(i);
}
void SgoManager::removeSgo(int index)
{
  assert(index >= 0 && index < sgoList.getLength() && "Index out of bounds.");
  SgObject *sgo = sgoList[index];
  SoSeparator *graph = sgo->getSceneGraph();
  assert(graph && "Object has not associated its scene graph.");
  sgoRoot->removeChild(graph);
  sgoList.remove(index);
  sgo->setManager(NULL);
}
void SgoManager::deleteSgo(int index)
{
  assert(index >= 0 && index < sgoList.getLength() && "Index out of bounds.");
  SgObject *sgo = sgoList[index];
  removeSgo(index);
  delete sgo;
}
void SgoManager::deleteSgo(SgObject *sgo)
{
  int i = sgoList.find(sgo);
  assert(i != -1 && "Object not present.");
  deleteSgo(i);
}
void SgoManager::deteleAllSgos()
{
  int i = sgoList.getLength();
  while (--i >= 0)
    deleteSgo(i);
}
SgoManager::CollidingType SgoManager::checkCollision()
{	//zjistuju kolize
  for (int i=0; i< sgoList.getLength(); i++){
	sgoList[i]->setForce(SbVec3f(0,0,0));
    if (sgoList[i]->getType()!=SgObject::SUN) sgoList[i]->setID(99); // mimo slunecni soustavu
  }
  CollidingType type=this->Null;
  for (int i=0; i< sgoList.getLength(); i++){ // pro kazdy objekt
	type=this->checkCollisionWith(sgoList[i],i+1);
	if(type!=this->Null){ //s kazdym dalsim objektem
	  this->CollidingSgos.Index[0]=i;	// jestlize kolidoval ulozim prvni ze zjistovanych objektu
	  return type;					// druhy se uklada primo ve funkci
	}
  }
  return type;
}
SgoManager::CollidingType SgoManager::checkCollisionWith(SgObject * so, int index)
{
  // FIXME: There is one major bug in this function - it detects
  // internal collisions in the scene objects. This usually happens
  // if the object is composed of more shape objects. PCJohn 2005-01-01

  // Improvement suggestions:
  // - generator of bounding box models
  // - generator of single-shape collision models
  // (- filtering functionality going around self-collisions)
  // (- more sofisticated collision algorithms)
  
  int i,c = sgoList.getLength();
  SbVec3f s,diff,force;
  double diffQRT, Lenght;//vzdalenost teles na druhou
  double rr,r1,r2; //polomer obou teles nadruhou, polomer telesa
  bool sun=false;
  vector<int> collIndex(0);
  vector<SgoManager::CollidingType> collType(0);
  
  // rozdeleni koliznich oblasti na slunecni soustavy
  for (i=index; i<c; i++) if (sgoList[i] != so){
	diff=s=so->getPos()-sgoList[i]->getPos();
	Lenght=s.normalize();
	force = s*(float)((G*(so->getMass()*sgoList[i]->getMass()))/(Lenght*Lenght));
	so->setForce(so->getForce()-force);
	sgoList[i]->setForce(sgoList[i]->getForce()+force);
	diffQRT=diff[0]*diff[0]+diff[1]*diff[1]+diff[2]*diff[2];
	if ((so->getType()==SgObject::SUN)||(sgoList[i]->getType()==SgObject::SUN)){//jestlize jeden z objektu je slunce
	  sun=true;
	  if (so->getType()==SgObject::SUN) r1=so->getActionRadius();
	  else r1=so->getCollisionRadius();
	  if (sgoList[i]->getType()==SgObject::SUN) r2=sgoList[i]->getActionRadius();
	  else r2=sgoList[i]->getCollisionRadius();
	  rr=(r1+r2)*(r1+r2);
      if (diffQRT<=rr){	//zjistim zda je druhy objekt v akcnim radiusu slunce
		if (so->getType()==SgObject::SUN) sgoList[i]->setID(so->getID());
		if (sgoList[i]->getType()==SgObject::SUN) so->setID(so->getID());
	  }
	}
	if (so->getID()==sgoList[i]->getID()){		// pro vsechny objekty ve stejne SS (nebo mimo vsechny SS)
	  r1=so->getCollisionRadius();
	  r2=sgoList[i]->getCollisionRadius();
      rr = (r1+r2)*(r1+r2); //umocneny soucet koliznich radiusu
	  if (diffQRT<=rr){ // kdyz koliduji koule
		collType.push_back(this->Body);
		if (sun) collType.back()=this->Explosion;
		if ((so->getType()==SgObject::PLANET)||(sgoList[i]->getType()==SgObject::PLANET)) collType.back()=this->Sphere;
		if ((so->getType()==SgObject::CAMERA)||(sgoList[i]->getType()==SgObject::CAMERA)) collType.back()=this->Sphere;
	    collIndex.push_back(i);	//pridam do seznamu na podrobnejsi kolizi
	  }
	}
  }
  
  if (collIndex.size()){	// collType ==Body
    assert(((SoGroup*)collisionRoot->getChild(0))->getNumChildren() == 0);
    assert(((SoGroup*)collisionRoot->getChild(1))->getNumChildren() == 0);
	this->CollidingSgos.Collision=false;
    ((SoGroup*)collisionRoot->getChild(0))->addChild(so->getMatrixTransform());
	if (so->getType()==SgObject::OTHER) ((SoGroup*)collisionRoot->getChild(0))->addChild(so->getCollisionModel());
    while (collIndex.size()){
	  if (sgoList[collIndex.back()] != so) {
	    if (collType.back()==this->Sphere){
		  //obsluha kolize se sphere
			((SoGroup*)collisionRoot->getChild(0))->removeAllChildren();
            ((SoGroup*)collisionRoot->getChild(1))->removeAllChildren();
			this->CollidingSgos.Index[1]=collIndex.back();
		  return collType.back();
	    }
		if (collType.back()==this->Explosion){
		  //obsluha kolize s explozi
            ((SoGroup*)collisionRoot->getChild(0))->removeAllChildren();
            ((SoGroup*)collisionRoot->getChild(1))->removeAllChildren();
			this->CollidingSgos.Index[1]=collIndex.back();
		  return collType.back();
	    }
		if (collType.back()==this->Body) {
          ((SoGroup*)collisionRoot->getChild(1))->addChild(sgoList[collIndex.back()]->getMatrixTransform());
          ((SoGroup*)collisionRoot->getChild(1))->addChild(sgoList[collIndex.back()]->getCollisionModel());
          ida->apply(collisionRoot);
		  if (this->CollidingSgos.Collision) {
            ((SoGroup*)collisionRoot->getChild(0))->removeAllChildren();
            ((SoGroup*)collisionRoot->getChild(1))->removeAllChildren();
		    this->CollidingSgos.Index[1]=collIndex.back();
			return collType.back();
          }
          ((SoGroup*)collisionRoot->getChild(1))->removeAllChildren();
        }
	    collIndex.pop_back();
	  }
	}
  }
  ((SoGroup*)collisionRoot->getChild(0))->removeAllChildren();
  return this->Null;
}
void SgoManager::resolveCollisionExplozion(const double time, const double dt){
  int first=this->CollidingSgos.Index[0];
  int second=this->CollidingSgos.Index[1];
  if ((this->sgoList[first]->getType()==SgObject::CAMERA)||(this->sgoList[second]->getType()==SgObject::CAMERA)){
	this->resolveCollisionSphere();
  }else{
	cout<<"Explozion !!!"<<endl;
	int k=0;
	int expIndex=0;
	if ((first==this->CameraFighter)||(second==this->CameraFighter)){
		for (int i=0; i<this->sgoList.getLength(); i++) 
			if (this->sgoList[i]->getType()==SgObject::OTHER){
				this->CameraFighter=i;
				this->setActiveFighter();
			}else this->setActiveFree();
	}
	if (this->sgoList[first]->getType()==SgObject::OTHER) expIndex=first;
	else expIndex=second;
	while((k<this->sgoBillboard.getLength())&&(this->sgoBillboard[k]->isUsed())) k++;
	if (k<this->sgoBillboard.getLength()){	// je volny billboard
		this->sgoBillboard[k]->setPos(this->sgoList[expIndex]->getPos());
		this->sgoBillboard[k]->setDimension(this->sgoList[expIndex]->getCollisionRadius()*2);
		this->sgoBillboard[k]->useBillboard();
		sgoRoot->addChild(this->sgoBillboard[k]->getSceneGraph());
	}else{									// neni volny billboard
		Billboard * b = new Billboard;
		b->setDimension(this->sgoList[expIndex]->getCollisionRadius()*2);
		//b->setTexture("test.jpg",8);
		b->setTexture("test.tga",4);
		b->setTextureFPS(10);
		b->setPos(this->sgoList[expIndex]->getPos());
		b->setCamera(this->Camera);
		b->useBillboard();
		b->setManager(this->sgoRoot);
		b->timeStep(time,dt);
		sgoRoot->addChild(b->getSceneGraph());
		this->sgoBillboard.append(b);
	}
	this->deleteSgo(expIndex);
  }
}
void SgoManager::resolveCollisionSphere(){
	cout<<"spere !!!"<<endl;
	SbVec3f axis,U1x,U1y,U2x,U2y, V1x,V1y,V2x,V2y;
    int first=this->CollidingSgos.Index[0];
    int second=this->CollidingSgos.Index[1];
//	cout<<first<<endl;
//	cout<<second<<endl;
	float M1=this->sgoList[first]->getMass();
	float M2=this->sgoList[second]->getMass();
	SbVec3f U1=this->sgoList[first]->getSpeed();
	SbVec3f U2=this->sgoList[second]->getSpeed();
	axis=this->sgoList[second]->getPos()-this->sgoList[first]->getPos();
	axis.normalize();
	U1x=axis*axis.dot(U1);
	U1y=U1-U1x;
	U2x=-axis*-axis.dot(U2);
	U2y=U2-U2x;
	V1x=((U1x*M1)+(U2x*M2)-(U1x-U2x)*M2)/(M1+M2);
	V2x=((U1x*M1)+(U2x*M2)-(U2x-U1x)*M1)/(M1+M2);
	V1y = U1y;
	V2y = U2y;
	/*cout<<U1[0]<<" , "<<U1[0]<<" , "<<U1[0]<<endl;
	cout<<U2[0]<<" , "<<U2[0]<<" , "<<U2[0]<<endl;
	cout<<(V1x+V1y)[0]<<" , "<<(V1x+V1y)[0]<<" , "<<(V1x+V1y)[0]<<endl;
	cout<<(V2x+V2y)[0]<<" , "<<(V2x+V2y)[0]<<" , "<<(V2x+V2y)[0]<<endl;*/
	this->sgoList[first]->setSpeed(V1x+V1y);
	this->sgoList[second]->setSpeed(V2x+V2y);
	sgoList[first]->setOldPos();
    sgoList[first]->setOldOrientation();
    sgoList[second]->setOldPos();
    sgoList[second]->setOldOrientation();
}

void SgoManager::resolveCollisionBody(){
	//compute impulse force
	int coeficientOfRestitution=1;
	SbVec3f Impulse(0,0,0);
	int first=this->CollidingSgos.Index[0];
	int second=this->CollidingSgos.Index[1];
	SbVec3f Point=this->CollidingSgos.collidingPoint;
	SbVec3f Normal=this->CollidingSgos.collidingNormal;
	SbVec3f R1=Point-this->sgoList[first]->getPos();
	SbVec3f R2=Point-this->sgoList[second]->getPos();
	SbMatrix I1=this->sgoList[first]->getInertiaTensor();
	SbMatrix I2=this->sgoList[second]->getInertiaTensor();
	float mass1=this->sgoList[first]->getMass();
	float mass2=this->sgoList[second]->getMass();
	SbVec3f v1=this->sgoList[first]->getVel();
	SbVec3f v2=this->sgoList[second]->getVel();
	SbVec3f omega1=this->sgoList[first]->getAngVel();
	SbVec3f omega2=this->sgoList[second]->getAngVel();
	SbVec3f v1omega=this->sgoList[first]->getAngMom();
	SbVec3f v2omega=this->sgoList[second]->getAngMom();

	SbVec3f vRel1=v1+omega1.cross(R1);
	SbVec3f vRel2=v2+omega2.cross(R2);
	SbVec3f vRel=vRel1-vRel2;
	float ImpulseNumerator=-(1+coeficientOfRestitution)*vRel.dot(Normal);

	/*cout<<"v rel1 "<<vRel1[0]<<" "<<vRel1[1]<<" "<<vRel1[2]<<endl;
	cout<<"v rel2 "<<vRel2[0]<<" "<<vRel2[1]<<" "<<vRel2[2]<<endl;
	cout<<"v relativni "<<vRel[0]<<" "<<vRel[1]<<" "<<vRel[2]<<endl;
	cout<<"Impulse Numerator "<<ImpulseNumerator<<endl<<endl;*/

	SbVec3f pomvec1=R1.cross(Normal);
	SbVec3f pomvec2=R2.cross(Normal);

	/*cout<<"R1xn "<<pomvec1[0]<<" "<<pomvec1[1]<<" "<<pomvec1[2]<<endl;
	cout<<"R2xn "<<pomvec2[0]<<" "<<pomvec2[1]<<" "<<pomvec2[2]<<endl;*/

	SbVec3f pom1(0,0,0);
	SbVec3f pom2(0,0,0);

	I1.multDirMatrix(pomvec1,pom1);
	I2.multDirMatrix(pomvec2,pom2);
	/*cout<<"pom1 "<<pom1[0]<<" "<<pom1[1]<<" "<<pom1[2]<<endl;
	cout<<"pom2 "<<pom2[0]<<" "<<pom2[1]<<" "<<pom2[2]<<endl;*/
	I1.multVecMatrix(pomvec1,pom1);
	I2.multVecMatrix(pomvec2,pom2);
	/*cout<<"pom1 "<<pom1[0]<<" "<<pom1[1]<<" "<<pom1[2]<<endl;
	cout<<"pom2 "<<pom2[0]<<" "<<pom2[1]<<" "<<pom2[2]<<endl;*/
	I1.multMatrixVec(pomvec1,pom1);
	I2.multMatrixVec(pomvec2,pom2);
	/*cout<<"pom1 "<<pom1[0]<<" "<<pom1[1]<<" "<<pom1[2]<<endl;
	cout<<"pom2 "<<pom2[0]<<" "<<pom2[1]<<" "<<pom2[2]<<endl;*/

	I1.multMatrixVec(pomvec1,pomvec1);
	I2.multMatrixVec(pomvec2,pomvec2);
	pomvec1=pomvec1.cross(R1);
	pomvec2=pomvec2.cross(R2);

	/*cout<<"pomvec1 "<<pomvec1[0]<<" "<<pomvec1[1]<<" "<<pomvec1[2]<<endl;
	cout<<"pomvec2 "<<pomvec2[0]<<" "<<pomvec2[1]<<" "<<pomvec2[2]<<endl;*/

    SbVec3f pom=(pomvec1+pomvec2);
	cout<<"pom "<<pom[0]<<" "<<pom[1]<<" "<<pom[2]<<endl;
	float soucin=pom.dot(Normal);
	//cout<<"soucin "<<soucin<<endl;
	float OneOwerMass=(1.f/mass1)+(1.f/mass2);
	//cout<<"oneovermas "<<OneOwerMass<<endl;
	float ImpulseDenominator=soucin+OneOwerMass;

	Impulse=(ImpulseNumerator/ImpulseDenominator)*Normal;
	
	/*cout<<"Impulse Numerator "<<ImpulseNumerator<<endl;	
	cout<<"Impulse Denominator "<<ImpulseDenominator<<endl;
	cout<<"Impulse "<<Impulse[0]<<" "<<Impulse[1]<<" "<<Impulse[2]<<endl;
		
	cout<<"Rychlost1-old: "<<v1[0]<<" "<<v1[1]<<" "<<v1[2]<<endl;
	cout<<"Rychlost2-old: "<<v2[0]<<" "<<v2[1]<<" "<<v2[2]<<endl;*/
	this->sgoList[first]->setVel(v1+Impulse/mass1);
	v1=this->sgoList[first]->getVel();
	this->sgoList[second]->setVel(v2-Impulse/mass2);
	v2=this->sgoList[second]->getVel();
	/*cout<<"Rychlost1:     "<<v1[0]<<" "<<v1[1]<<" "<<v1[2]<<endl;
	cout<<"Rychlost2:     "<<v2[0]<<" "<<v2[1]<<" "<<v2[2]<<endl;
		

	cout<<"omega1-old: "<<omega1[0]<<" "<<omega1[1]<<" "<<omega1[2]<<endl;
	cout<<"omega2-old: "<<omega2[0]<<" "<<omega2[1]<<" "<<omega2[2]<<endl;*/
	float f1=omega1.length();
	float f2=omega2.length();
	/*cout<<"Omega1-mag: "<<f1<<endl;
	cout<<"Omega2-mag: "<<f2<<endl;*/
	SbVec3f pomAM1=v1omega+R1.cross(Impulse);
	SbVec3f pomAM2=v2omega+R2.cross(-Impulse);
	this->sgoList[first]->setAngMom(pomAM1);
	this->sgoList[second]->setAngMom(pomAM2);
	I1.multDirMatrix(pomAM1,omega1);
	I2.multDirMatrix(pomAM2,omega2);
	this->sgoList[first]->setAngVel(omega1);
	this->sgoList[second]->setAngVel(omega2);
	/*cout<<"omega1:     "<<omega1[0]<<" "<<omega1[1]<<" "<<omega1[2]<<endl;
	cout<<"omega2:     "<<omega2[0]<<" "<<omega2[1]<<" "<<omega2[2]<<endl;*/
	f1=omega1.length();
	f2=omega2.length();
	/*cout<<"Omega1-mag: "<<f1<<endl;
	cout<<"Omega2-mag: "<<f2<<endl;
	cout<<endl;*/
}


SoIntersectionDetectionAction::Resp SgoManager::intersectionCb(void * closure,
    const SoIntersectingPrimitive *p1, const SoIntersectingPrimitive *p2)
{
  SgoManager * THIS = (SgoManager*)closure;		// ukazatel na class manager
  THIS->CollidingSgos.Collision=true;					// nastaveni priznaku kolize
	// pomocne promene pro vypocet bodu kolize
  SbVec3f n1,n2,r,diff,point;
  SbVec3f p1a=p1->xf_vertex[0];
  SbVec3f p1b=p1->xf_vertex[1];
  SbVec3f p2a=p2->xf_vertex[0];
  SbVec3f p2b=p2->xf_vertex[1];
  n1=p1a.cross(p1b);	//normala pro rovinu p1
  n2=p2b.cross(p2a);	//normala pro rovinu p2
  vector<SbVec3f> points; // vypoctene pruseciky
  float t=0;	// pomocna promena pro vypocet pruseciku
  float den=0;	// denominator pro vypocet t

  for (int i=0; i<3; i++) {
    r=p1->xf_vertex[(i+1)%3]-p1->xf_vertex[(i)%3];
	den=n2.dot(r);
	if (den!=0){
		t=-n2.dot(p1->xf_vertex[i%3])/n2.dot(r);
		point=SbVec3f((p1->xf_vertex[i%3][0]+r[0]*t),(p1->xf_vertex[i%3][1]+r[1]*t),(p1->xf_vertex[i%3][2]+r[2]*t));
		bool neni=true;
		for (unsigned int j=0; j<points.size(); j++) if (THIS->diffExp2(points[j],point)<=THIS->epsilon) neni=false;
		if (neni)points.push_back(point);
		else cout<<"v kolizi se vyskyli stejne body"<<endl;
	}else cout<<"hrana trojuhelniku je rovnobezna s rovinou"<<endl;
  }
  for (int i=0; i<3; i++) {
    r=p2->xf_vertex[(i+1)%3]-p2->xf_vertex[(i)%3];
	den=n1.dot(r);
	if (den!=0){
		t=-n1.dot(p2->xf_vertex[i%3])/n1.dot(r);
		point=SbVec3f((p2->xf_vertex[i%3][0]+r[0]*t),(p2->xf_vertex[i%3][1]+r[1]*t),(p2->xf_vertex[i%3][2]+r[2]*t));
		bool neni=true;
		for (unsigned int j=0; j<points.size(); j++) if (THIS->diffExp2(points[j],point)<=THIS->epsilon) neni=false;
		if (neni) points.push_back(point);
		else cout<<"v kolizi se vyskyli stejne body"<<endl;
	}else cout<<"hrana trojuhelniku je rovnobezna s rovinou druheho trojuhelniku"<<endl;
  }
  //for (int i=0;i<points.size();i++) cout<<points[i][0]<<" "<<points[i][1]<<" "<<points[i][2]<<endl;
  while (points.size()){ // prirazeni vsech koliznich bodu do vektoru
	  THIS->CollidingSgos.Points.push_back(points.back());
	  points.pop_back();
  }
  //cout<<"&"<<endl;
  if (p1->path->getIndex(1)){
	THIS->CollidingSgos.Triangles[0].push_back(p1->xf_vertex[0]);
	THIS->CollidingSgos.Triangles[0].push_back(p1->xf_vertex[1]);
	THIS->CollidingSgos.Triangles[0].push_back(p1->xf_vertex[2]);
  }else{
	THIS->CollidingSgos.Triangles[1].push_back(p1->xf_vertex[0]);
	THIS->CollidingSgos.Triangles[1].push_back(p1->xf_vertex[1]);
	THIS->CollidingSgos.Triangles[1].push_back(p1->xf_vertex[2]);
  }
  if (p2->path->getIndex(1)){
	THIS->CollidingSgos.Triangles[0].push_back(p2->xf_vertex[0]);
	THIS->CollidingSgos.Triangles[0].push_back(p2->xf_vertex[1]);
	THIS->CollidingSgos.Triangles[0].push_back(p2->xf_vertex[2]);
  }else{
	THIS->CollidingSgos.Triangles[1].push_back(p2->xf_vertex[0]);
	THIS->CollidingSgos.Triangles[1].push_back(p2->xf_vertex[1]);
	THIS->CollidingSgos.Triangles[1].push_back(p2->xf_vertex[2]);
  }
  return SoIntersectionDetectionAction::NEXT_PRIMITIVE;
}

float SgoManager::diffExp2(SbVec3f v1, SbVec3f v2){
  SbVec3f pom=v1-v2;
  return pom[0]*pom[0]+pom[1]*pom[1]+pom[2]*pom[2];
}
void SgoManager::workIntersectionPoints(){
  vector<SbVec3f> pomP;
  for (unsigned int i=0;i<CollidingSgos.Points.size()-1;i++){
    bool dual=false;
	for (unsigned int j=i+1;j<CollidingSgos.Points.size();j++)
	  if (diffExp2(CollidingSgos.Points[i],CollidingSgos.Points[j])<this->epsilon) dual=true;
    if (dual) pomP.push_back(CollidingSgos.Points[i]);
  }
  while (CollidingSgos.Points.size()) CollidingSgos.Points.pop_back();
  for (unsigned int i=0;i<pomP.size();i++) CollidingSgos.Points.push_back(pomP[i]);
}
void SgoManager::workIntersectionVertexs(){
	vector<SbVec3f> pomP;
	bool a,b,c;
	//cout<<"pocet trojuhelniku "<<this->CollidingSgos.Triangles[0].size()/3+this->CollidingSgos.Triangles[1].size()/3<<endl;
	for (int k=0;k<2;k++){
		pomP.resize(0);
		for(int i=0;i<this->CollidingSgos.Triangles[k].size();i+=3){
			for (int j=i+3;j<this->CollidingSgos.Triangles[k].size();j+=3){
				a=b=c=false;
				for (int l=0;l<3;l++){
					if (this->CollidingSgos.Triangles[k][i+0]==this->CollidingSgos.Triangles[k][j+l]) a=true;
					if (this->CollidingSgos.Triangles[k][i+1]==this->CollidingSgos.Triangles[k][j+l]) b=true;
					if (this->CollidingSgos.Triangles[k][i+2]==this->CollidingSgos.Triangles[k][j+l]) c=true;
				}
				if (a&&b&&c) {
					this->CollidingSgos.Triangles[k][j+0]=SbVec3f(-999,-999,-999);
					this->CollidingSgos.Triangles[k][j+1]=SbVec3f(-999,-999,-999);
					this->CollidingSgos.Triangles[k][j+2]=SbVec3f(-999,-999,-999);
				}
			}
		}
		for(int i=0;i<this->CollidingSgos.Triangles[k].size();i++)
			if (this->CollidingSgos.Triangles[k][i][0]!=-999) pomP.push_back(this->CollidingSgos.Triangles[k][i]);
		while (this->CollidingSgos.Triangles[k].size()) this->CollidingSgos.Triangles[k].pop_back();
		for(int i=0;i<pomP.size();i++)this->CollidingSgos.Triangles[k].push_back(pomP[i]);
	}
	cout<<"Prvni objekt, pocet troj. "<<this->CollidingSgos.Triangles[0].size()/3<<endl;
	cout<<"Druhy objekt, pocet troj. "<<this->CollidingSgos.Triangles[1].size()/3<<endl;
	//	cout<<"pole "<<endl;
//	for(int i=0;i<index.size();i++)	cout<<index[i]<<endl;
}

SbVec3f SgoManager::getIntersectionPoint(){
  SbVec3f CP(0,0,0);
  for (unsigned int i=0;i<CollidingSgos.Points.size();i++) CP+=CollidingSgos.Points[i];
  CP/=(const float)CollidingSgos.Points.size();
  this->CollidingSgos.collidingPoint=CP;
  return CP;
}
void SgoManager::computeNormal(){
	  int index=0;
	  SbVec3f normala(0,0,0);
	  if (this->CollidingSgos.Triangles[0].size()<this->CollidingSgos.Triangles[0].size())index=0;
	  else index=1;
	  if (this->CollidingSgos.Triangles[index].size()==3){
/*			cout<<"trojuhelnik "<<this->CollidingSgos.Triangles[index][0][0]<<" "<<this->CollidingSgos.Triangles[index][0][1]<<" "<<this->CollidingSgos.Triangles[index][0][2]<<endl;
			cout<<"trojuhelnik "<<this->CollidingSgos.Triangles[index][1][0]<<" "<<this->CollidingSgos.Triangles[index][1][1]<<" "<<this->CollidingSgos.Triangles[index][1][2]<<endl;
			cout<<"trojuhelnik "<<this->CollidingSgos.Triangles[index][2][0]<<" "<<this->CollidingSgos.Triangles[index][2][1]<<" "<<this->CollidingSgos.Triangles[index][2][2]<<endl;*/
			SbVec3f a=this->CollidingSgos.Triangles[index][0]-this->CollidingSgos.Triangles[index][1];
			SbVec3f b=this->CollidingSgos.Triangles[index][1]-this->CollidingSgos.Triangles[index][2];
			normala=a.cross(b);
			normala.normalize();
			this->CollidingSgos.collidingNormal=normala;
	  } else {
		  for (int i=0;i<this->CollidingSgos.Triangles[index].size()-1;i+=3){
			  SbVec3f pom=this->CollidingSgos.Triangles[index][i].cross(this->CollidingSgos.Triangles[index][i+1]);
			  pom.normalize();
			  normala+=pom;
		  }
		  normala.normalize();
		  if (index) normala*=-1;
		  this->CollidingSgos.collidingNormal=normala;
	  }
}
void SgoManager::timeStep(const double time, const double dt)
{
  SbList<SgObject*> tmp = sgoList;
  int c = tmp.getLength();

  for (int i=0; i<c; i++)
    // Following condition is necessary, because any sgo can
    // remove itself or other sgos from the list during the execution
    // of the following call to SgObject::timeStep(time, dt).
    if (sgoList.find(tmp[i]) != -1)	tmp[i]->timeStep(time, dt);
  
  for (int i=0; i<this->sgoBillboard.getLength(); i++)
	  if (this->sgoBillboard[i]->isUsed()) 
		  this->sgoBillboard[i]->timeStep(time,dt);
	  //else sgoRoot->removeChild(this->sgoBillboard[i]->getSceneGraph());

  CollidingType type=this->checkCollision();
  
  if (type==this->Explosion){
	  this->resolveCollisionExplozion(time,dt);
  }
  if (type==this->Sphere){
	  this->resolveCollisionSphere();
  }
  if (type==this->Body){
	  //resolve collision
	  //	  cout<<"pocet prvniho"<<this->CollidingSgo.firstTriangles.size()/3.f<<endl;
//	  cout<<"pocet druheho"<<this->CollidingSgo.secondTriangles.size()/3.f<<endl;
//	  cout<<"pocet koliznich bodu"<<this->CollidingSgos.Points.size()<<endl;
	  cout<<"body !!!"<<endl;
	  this->workIntersectionPoints();
	  SbVec3f CM = this->getIntersectionPoint();
	  this->workIntersectionVertexs();
	  this->computeNormal();
	  this->resolveCollisionBody();
  	  sgoList[this->CollidingSgos.Index[0]]->setOldPos();
	  sgoList[this->CollidingSgos.Index[0]]->setOldOrientation();
	  sgoList[this->CollidingSgos.Index[1]]->setOldPos();
	  sgoList[this->CollidingSgos.Index[1]]->setOldOrientation();
	  cout<<"konec"<<endl;
  }
}
